@@ -33,6 +33,9 @@
  * In the future, automatic saving to GConf or .ini would be such
  * feature easily added with this design.
  **/
+
+#include <glib/gi18n.h>
+#include <gconf/gconf-client.h>
 #include "gnome-scan-settings.h"
 
 #define	GET_PRIVATE(o)	(G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOME_TYPE_SCAN_SETTINGS, GnomeScanSettingsPrivate))
@@ -41,8 +44,10 @@ typedef struct _GnomeScanSettingsPrivate GnomeScanSettingsPrivate;
 
 struct _GnomeScanSettingsPrivate
 {
-  /* using GParamSpec name as key, and GValue as value */
-  GHashTable	*values;
+	/* using GParamSpec name as key, and GValue as value */
+	GHashTable	*values;
+	GConfClient *gconf;
+	gboolean	disposed;
 };
 
 enum
@@ -59,27 +64,223 @@ G_DEFINE_TYPE (GnomeScanSettings, gnome_scan_settings, G_TYPE_OBJECT);
 static void
 gnome_scan_settings_init (GnomeScanSettings *object)
 {
-  GnomeScanSettingsPrivate *priv = GET_PRIVATE (object);
+	GnomeScanSettingsPrivate *priv = GET_PRIVATE (object);
 	
-  priv->values = g_hash_table_new (g_str_hash, g_str_equal);
+	priv->values = g_hash_table_new (g_str_hash, g_str_equal);
+	priv->gconf = gconf_client_get_default();
+}
+
+static gchar*
+gss_gconf_key(gchar *name)
+{
+	gchar *key;
+	gchar *tmp = g_strdup_printf ("/apps/gnome-scan/%s", g_get_prgname ());
+	
+	if (name)
+		key = g_strdup_printf("%s/%s", tmp, name);
+	else
+		key = g_strdup(tmp);
+		
+	g_free(tmp);
+	
+	return key;
+}
+
+static GValue*
+gss_gconf_value_to_gvalue(GConfValue *value)
+{
+	GType gtype = G_TYPE_NONE;
+	GValue *gvalue = g_new0(GValue, 1);
+	
+	switch(value->type) {
+		case GCONF_VALUE_STRING:
+			gtype = G_TYPE_STRING;
+			break;
+		case GCONF_VALUE_INT:
+			gtype = G_TYPE_INT;
+			break;
+		case GCONF_VALUE_FLOAT:
+			gtype = G_TYPE_FLOAT;
+			break;
+		case GCONF_VALUE_BOOL:
+			gtype = G_TYPE_BOOLEAN;
+			break;
+	}
+	
+	g_value_init(gvalue, gtype);
+	
+	switch(value->type) {
+		case GCONF_VALUE_STRING:
+			g_value_set_static_string(gvalue,
+									  gconf_value_get_string(value));
+			break;
+		case GCONF_VALUE_INT:
+			g_value_set_int(gvalue,
+							gconf_value_get_int(value));
+			break;
+		case GCONF_VALUE_FLOAT:
+			g_value_set_float(gvalue,
+							  gconf_value_get_float(value));
+			break;
+		case GCONF_VALUE_BOOL:
+			g_value_set_boolean(gvalue,
+								gconf_value_get_bool(value));
+			break;
+	}
+	
+	if (gtype == G_TYPE_NONE) {
+		g_free(gvalue);
+		gvalue = NULL;
+	}
+	
+	return gvalue;
+}
+
+static GConfValue*
+gss_gvalue_to_gconf_value(GValue *gvalue)
+{
+	GConfValueType type = GCONF_VALUE_INVALID;
+	GConfValue *value;
+	GValue *trans = g_new0(GValue, 1);
+	GType gtype = G_VALUE_TYPE(gvalue);
+	switch(gtype) {
+		case G_TYPE_STRING:
+			type = GCONF_VALUE_STRING;
+			break;
+		case G_TYPE_INT:
+			type = GCONF_VALUE_INT;
+			break;
+		case G_TYPE_FLOAT:
+		case G_TYPE_DOUBLE:
+			type = GCONF_VALUE_FLOAT;
+			break;
+		case G_TYPE_BOOLEAN:
+			type = GCONF_VALUE_BOOL;
+			break;
+	}
+
+	value = gconf_value_new(type);
+	
+	switch(gtype) {
+		case G_TYPE_STRING:
+			gconf_value_set_string(value,
+								   g_value_get_string(gvalue));
+			break;
+		case G_TYPE_INT:
+			gconf_value_set_int(value,
+								g_value_get_int(gvalue));
+			break;
+		case G_TYPE_FLOAT:
+		case G_TYPE_DOUBLE:
+			g_value_init(trans, G_TYPE_FLOAT);
+			g_value_transform (gvalue, trans);
+			gconf_value_set_int(value,
+								g_value_get_float(trans));
+			break;
+		case G_TYPE_BOOLEAN:
+			gconf_value_set_bool(value,
+								 g_value_get_boolean (gvalue));
+			break;
+	}
+	
+	if (!GCONF_VALUE_TYPE_VALID(type)) {
+		g_free(value);
+		value = NULL;
+	}
+	
+	g_free(trans);
+	
+	return value;
+}
+
+static GObject*
+gnome_scan_settings_constructor (GType type,
+								 guint n_params,
+								 GObjectConstructParam *params)
+{
+	GObject *object;
+	GnomeScanSettingsPrivate *priv;
+	GSList *entries;
+	gchar *dir = gss_gconf_key(NULL);
+	gchar *name;
+	GError *error = NULL;
+	GConfEntry *entry;
+	GValue *gvalue;
+	
+	object =
+		G_OBJECT_CLASS(parent_class)->constructor(type,
+												  n_params,
+												  params);
+	priv = GET_PRIVATE(object);
+	entries = gconf_client_all_entries(priv->gconf, dir, &error);
+	for (; entries; entries = entries->next) {
+		entry = entries->data;
+		name = g_path_get_basename(entry->key);
+		if (gvalue = gss_gconf_value_to_gvalue(entry->value))
+				gnome_scan_settings_set(GNOME_SCAN_SETTINGS(object),
+										name, gvalue);
+		g_free(name);
+	}
+	
+	g_free(dir);
+	
+	return object;
 }
 
 static void
 gnome_scan_settings_finalize (GObject *object)
 {
-  /* TODO: Add deinitalization code here */
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+void
+gss_store_key(gchar *name, GValue *gvalue, GnomeScanSettings *settings)
+{
+	GnomeScanSettingsPrivate *priv = GET_PRIVATE(settings);
+	GConfValue *value;
+	gchar *key = gss_gconf_key(name);
+	GError *error = NULL;
+	
+	if (value = gss_gvalue_to_gconf_value(gvalue)) {
+		gconf_client_set(priv->gconf,
+						 key, value,
+						 &error);
+
+		if (error) {
+			g_warning("Failed saving %s option : %s",
+					  name, dgettext(g_quark_to_string (error->domain),
+									 error->message));
+		}
+	}
+
+	g_free(key);
+}
+
+static void
+gnome_scan_settings_dispose (GObject *object)
+{
+	GnomeScanSettingsPrivate *priv = GET_PRIVATE(object);
+	
+	if (priv->disposed == FALSE) {
+		g_hash_table_foreach (priv->values,
+							  (GHFunc) gss_store_key,
+							  object);
+		priv->disposed = TRUE;
+	}
+	
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
 gnome_scan_settings_class_init (GnomeScanSettingsClass *klass)
 {
-  GObjectClass* object_class = G_OBJECT_CLASS (klass);
-  parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
-
-  g_type_class_add_private (klass, sizeof (GnomeScanSettingsPrivate));
-  object_class->finalize = gnome_scan_settings_finalize;
+	GObjectClass* object_class = G_OBJECT_CLASS (klass);
+	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
+	
+	g_type_class_add_private (klass, sizeof (GnomeScanSettingsPrivate));
+	object_class->constructor   = gnome_scan_settings_constructor;
+	object_class->finalize		= gnome_scan_settings_finalize;
+	object_class->dispose		= gnome_scan_settings_dispose;
 	
   /**
    * GnomeScanParamWidget::changed:
@@ -117,17 +318,18 @@ gnome_scan_settings_new (void)
 
 void
 gnome_scan_settings_init_value (GnomeScanSettings *settings,
-				GParamSpec *pspec)
+								GParamSpec *pspec)
 {
-  g_return_if_fail (GNOME_IS_SCAN_SETTINGS (settings));
-  GValue *value = g_new0 (GValue, 1);
-  g_value_init (value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-  g_param_value_set_default (pspec, value);
-  gnome_scan_settings_set (settings,
-			   g_param_spec_get_name (pspec),
-			   value);
-  g_value_reset (value);
-  g_free (value);
+	/* TODO: store schemas in gconf */
+	g_return_if_fail (GNOME_IS_SCAN_SETTINGS (settings));
+	GValue *value = g_new0 (GValue, 1);
+	g_value_init (value, G_PARAM_SPEC_VALUE_TYPE (pspec));
+	g_param_value_set_default (pspec, value);
+	gnome_scan_settings_set (settings,
+							 g_param_spec_get_name (pspec),
+							 value);
+	g_value_reset (value);
+	g_free (value);
 }
 
 /**
@@ -144,10 +346,10 @@ gnome_scan_settings_set (GnomeScanSettings *settings,
                          const gchar *name,
                          const GValue *value)
 {
-  g_return_if_fail (GNOME_IS_SCAN_SETTINGS (settings) && G_IS_VALUE (value));
-  g_hash_table_insert (GET_PRIVATE (settings)->values, g_strdup (name),
-                       g_boxed_copy (G_TYPE_VALUE, value));
-  g_signal_emit (settings, signals[CHANGED], 0, name);
+	g_return_if_fail (GNOME_IS_SCAN_SETTINGS (settings) && G_IS_VALUE (value));
+	g_hash_table_insert (GET_PRIVATE (settings)->values, g_strdup (name),
+						 g_boxed_copy (G_TYPE_VALUE, value));
+	g_signal_emit (settings, signals[CHANGED], 0, name);
 }
 
 /**
@@ -167,18 +369,18 @@ gnome_scan_settings_set_transform (GnomeScanSettings *settings,
 				   const gchar *name,
 				   const GValue *value)
 {
-  g_return_if_fail (GNOME_IS_SCAN_SETTINGS (settings) && G_IS_VALUE (value));
-  GValue *v = gnome_scan_settings_get (settings, name);
-  if (v) {
-    g_value_transform (value, v);
-  }
-  else {
-    v = g_boxed_copy (G_TYPE_VALUE, value);
-  }
-
-  g_hash_table_insert (GET_PRIVATE (settings)->values, g_strdup (name),
-		       v);
-  g_signal_emit (settings, signals[CHANGED], 0, name);
+	g_return_if_fail (GNOME_IS_SCAN_SETTINGS (settings) && G_IS_VALUE (value));
+	GValue *v = gnome_scan_settings_get (settings, name);
+	if (v) {
+		g_value_transform (value, v);
+	}
+	else {
+		v = g_boxed_copy (G_TYPE_VALUE, value);
+	}
+	
+	g_hash_table_insert (GET_PRIVATE (settings)->values, g_strdup (name),
+						 v);
+	g_signal_emit (settings, signals[CHANGED], 0, name);
 }
 
 /**
@@ -223,6 +425,27 @@ gnome_scan_settings_set_int (GnomeScanSettings *settings,
   gnome_scan_settings_set_transform (settings, name, v);
 }
 
+/**
+ * gnome_scan_settings_set_boolean:
+ * @settings: a #GnomeScanSettings
+ * @name: #GParamSpec name
+ * @value: gboolean value
+ * 
+ *
+ * See: gnome_scan_settings_set_transform()
+ **/
+void
+gnome_scan_settings_set_boolean (GnomeScanSettings *settings,
+                             const gchar *name,
+                             gboolean value)
+{
+  g_return_if_fail (GNOME_IS_SCAN_SETTINGS (settings));
+  GValue *v = g_new0 (GValue, 1);
+  g_value_init (v, G_TYPE_BOOLEAN);
+  g_value_set_int (v, value);
+  gnome_scan_settings_set_transform (settings, name, v);
+}
+
 /**
  * gnome_scan_settings_set_enum:
  * @settings: a #GnomeScanSettings
@@ -434,6 +657,19 @@ gnome_scan_settings_get_int (GnomeScanSettings *settings,
   return val;
 }
 
+gboolean
+gnome_scan_settings_get_boolean (GnomeScanSettings *settings,
+                             const gchar *name)
+{
+  g_return_val_if_fail (GNOME_IS_SCAN_SETTINGS (settings), 0);
+  GValue *v = gnome_scan_settings_get_transformed (settings,
+						   name,  G_TYPE_BOOLEAN);
+  gboolean val = g_value_get_boolean (v);
+  g_value_unset (v);
+  g_free (v);
+  return val;
+}
+
 /**
  * gnome_scan_settings_get_enum:
  * @settings: a #GnomeScanSettings
